# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
 <schema xmlns="http://www.w3.org/2001/XMLSchema"
 xmlns:xsd="http://www.w3.org/2001/XMLSchema"
    version="1.0" targetNamespace="http://www.example.com/test"
 xmlns:tgt="http://www.example.com/test"
    elementFormDefault="qualified" attributeFormDefault="unqualified">

  <complexType name="BaseRequestType" abstract="true">
    <attribute  name="attrib1"  type="xsd:string"/>
    <anyAttribute namespace="##any" processContents="skip"/>
  </complexType>

  <element name="Notification">
    <complexType>
      <complexContent>
        <extension base="tgt:BaseRequestType">
          <attribute name="attrib2" type="xsd:string" use="optional"/>
          <anyAttribute  namespace="##any" processContents="skip"/>
        </extension>
      </complexContent>
    </complexType>
  </element>
</schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0100 (unittest.TestCase):
    xmls_base = '<tgt:Notification xmlns:tgt="http://www.example.com/test" attrib1="text"/>'
    def testBasic (self):
        instance = CreateFromDocument(self.xmls_base)
        self.assertEqual(instance.attrib1, "text")

    xmls_wc = '<tgt:Notification xmlns:tgt="http://www.example.com/test" attrib2="text2" xmlns:other="urn:other" other:attrib3="text3"/>'
    def testWildcard (self):
        instance = CreateFromDocument(self.xmls_wc)
        self.assertEqual(instance.attrib1, None)
        self.assertEqual(instance.attrib2, "text2")
        wca = instance.wildcardAttributeMap()
        self.assertEqual(1, len(wca))
        (attr, val) = next(six.iteritems(wca))
        self.assertEqual(attr.namespaceURI(), "urn:other")
        self.assertEqual(attr.localName(), "attrib3")
        self.assertEqual(val, "text3")


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-empty-cstd.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestEmptyCSTD (unittest.TestCase):
    def testPresent (self):
        xmls = '<time xmlns="urn:test">http://test/something</time>'
        instance = CreateFromDocument(xmls)
        self.assertEqual("http://test/something", instance.value())
    def testMissing (self):
        xmls = '<time xmlns="urn:test"></time>'
        instance = CreateFromDocument(xmls)
        self.assertEqual("", instance.value())
    def testWhitespace (self):
        xmls = '<time xmlns="urn:test">   </time>'
        instance = CreateFromDocument(xmls)
        self.assertEqual(six.u(''), instance.value())
    def testEmpty (self):
        xmls = '<time xmlns="urn:test"/>'
        instance = CreateFromDocument(xmls)
        self.assertEqual("", instance.value())

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb
import pyxb.binding.generate
import pyxb.utils.domutils

from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/time.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

def make_tTime (*args, **kw):
    for cls in [ tXMTime, tISO8601 ]:
        try:
            v = cls(*args, **kw)
            v.validateBinding()
            return v
        except Exception as e:
            pass
    return None
tTime._SetAlternativeConstructor(make_tTime)

import unittest

class TestTime (unittest.TestCase):
    KW_tISO8601 = { 'time' : '2009-06-03T13:43:00Z' }
    KW_tXMTime = { 'seconds' : 2, 'fractionalSeconds' : 0.3 }
    def testXMTime (self):
        t = tXMTime(seconds=1)
        self.assertEqual(1, t.seconds)
        self.assertEqual(None, t.fractionalSeconds)
        t = tXMTime(**self.KW_tXMTime)
        self.assertEqual(2, t.seconds)
        self.assertEqual(0.3, t.fractionalSeconds)
        t._setElement(time)
        xmls = t.toxml("utf-8")
        instance = CreateFromDocument(xmls)
        self.assertEqual(instance.seconds, t.seconds)
        self.assertEqual(instance.fractionalSeconds, t.fractionalSeconds)

    def testISO8601 (self):
        t = tISO8601(**self.KW_tISO8601)
        self.assertEqual((2009, 6, 3, 13, 43, 0, 2, 154, 0), t.time.timetuple())
        t._setElement(time)
        xmls = t.toxml("utf-8")
        instance = CreateFromDocument(xmls)
        self.assertEqual(instance.time.timetuple(), t.time.timetuple())

    def testAbstract (self):
        self.assertRaises(pyxb.AbstractInstantiationError, tTime, **self.KW_tXMTime)
        t = make_tTime(**self.KW_tXMTime)
        self.assertTrue(isinstance(t, tTime))
        self.assertTrue(isinstance(t, tXMTime))
        t = make_tTime(**self.KW_tISO8601)
        self.assertTrue(isinstance(t, tTime))
        self.assertTrue(isinstance(t, tISO8601))
        t = tTime.Factory(**self.KW_tXMTime)
        self.assertTrue(isinstance(t, tTime))
        self.assertTrue(isinstance(t, tXMTime))



if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import pyxb.binding.basis

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/alt-po1.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

from pyxb.utils import domutils

def ToDOM (instance, dom_support=None):
    return instance.toDOM(dom_support).documentElement

import unittest

class TestProperties (unittest.TestCase):

    street_content = '''95 Main St.
Anytown, AS  12345-6789'''
    street_xmlt = six.u('<street>%s</street>') % (street_content,)
    street_xmld = street_xmlt.encode('utf-8')
    street_dom = pyxb.utils.domutils.StringToDOM(street_xmlt).documentElement

    address1_xmlt = six.u('<name>Customer</name><street>95 Main St</street>')
    address2_xmlt = six.u('<name>Sugar Mama</name><street>24 E. Dearling Ave.</street>')

    def testPythonElementSimpleContent (self):
        elt = USAddress._ElementMap['street'].elementBinding()(self.street_content)
        self.assertEqual(self.street_content, elt)
        self.assertEqual(ToDOM(elt).toxml("utf-8"), self.street_xmld)

    def testDOMElementSimpleContent (self):
        elt = USAddress._ElementMap['street'].elementBinding().createFromDOM(self.street_dom)
        self.assertEqual(ToDOM(elt).toxml("utf-8"), self.street_xmld)

    def testPythonElementComplexContent_Element (self):
        addr = USAddress(name='Customer', street='95 Main St')
        self.assertEqual('95 Main St', addr.street)
        addr = USAddress('Customer', '95 Main St')
        self.assertEqual('95 Main St', addr.street)
        addr.street = '43 West Oak'
        self.assertEqual('43 West Oak', addr.street)

    def testDOM_CTD_element (self):
        # NB: USAddress is a CTD, not an element.
        xmlt = six.u('<shipTo>%s</shipTo>') % (self.address1_xmlt,)
        dom = pyxb.utils.domutils.StringToDOM(xmlt)
        addr2 = USAddress.Factory(_dom_node=dom.documentElement)

    def testPurchaseOrder (self):
        po = purchaseOrder(shipTo=USAddress(name='Customer', street='95 Main St'),
                           billTo=USAddress(name='Sugar Mama', street='24 E. Dearling Ave'),
                           comment='Thanks!')
        xmld = ToDOM(po).toxml("utf-8")
        xml1t = '<ns1:purchaseOrder xmlns:ns1="http://www.example.com/altPO1"><shipTo><name>Customer</name><street>95 Main St</street></shipTo><billTo><name>Sugar Mama</name><street>24 E. Dearling Ave</street></billTo><ns1:comment>Thanks!</ns1:comment></ns1:purchaseOrder>'
        xml1d = xml1t.encode('utf-8')
        self.assertEqual(xmld, xml1d)

        dom = pyxb.utils.domutils.StringToDOM(xml1t)
        po2 = purchaseOrder.createFromDOM(dom.documentElement)
        self.assertEqual(ToDOM(po2).toxml("utf-8"), xml1d)

        xml2t = '<purchaseOrder xmlns="http://www.example.com/altPO1"><shipTo><name>Customer</name><street>95 Main St</street></shipTo><billTo><name>Sugar Mama</name><street>24 E. Dearling Ave</street></billTo><comment>Thanks!</comment></purchaseOrder>'
        xml2d = xml2t.encode('utf-8')
        bds = pyxb.utils.domutils.BindingDOMSupport()
        bds.setDefaultNamespace(Namespace)
        self.assertEqual(ToDOM(po2, dom_support=bds).toxml("utf-8"), xml2d)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import time
import sys
import DWML
import datetime
import pyxb
from pyxb.utils import domutils, six
from pyxb.utils.six.moves.urllib import request as urllib_request
import pyxb.binding.datatypes as xsd
import pyxb.bundles.wssplat.soap11 as soapenv

today = xsd.dateTime.today()
later = today + datetime.timedelta(days=7)

# Set a standard position for which we want the weather
lat = 38.898748
lon = -77.037684

# Override the position from the command line
if 2 < len(sys.argv):
    lat = float(sys.argv[1])
    lon = float(sys.argv[2])

# Import the schema bindings that were extracted from the WSDL in a
# separate, previous step.
import ndfd

# Read in the WSDL spec for the service.  Note that we have to process
# the schema again here, because we were unable to save the component
# model for it before, and we need the definition maps in order to
# resolve part type references in the WSDL messages.
import pyxb.bundles.wssplat.wsdl11 as wsdl
uri_src = open('ndfdXML.wsdl')
doc = domutils.StringToDOM(uri_src.read())
spec = wsdl.definitions.createFromDOM(doc.documentElement, process_schema=True)

# Create a helper that will generate XML in the WSDL's namespace,
# qualifying every element with xsi:type just like the service
# expects.
bds = domutils.BindingDOMSupport(default_namespace=spec.targetNamespace(), require_xsi_type=True)

# Set the parameters that you want enabled.  See
# http://www.nws.noaa.gov/xml/docs/elementInputNames.php
weather_params = ndfd.weatherParametersType(maxt=True, mint=True, temp=True, sky=True, pop12=True, rh=True, wx=True, appt=True)

# The schema didn't say the other parameters are optional (even though
# they are), so set them to false if not already initialized.
for eu in six.itervalues(weather_params._ElementMap):
    if eu.value(weather_params) is None:
        eu.set(weather_params, False)

# There is no schema element or type corresponding to the request
# message; it's only in a WSDL message definition.  We need to build
# it manually.

# Create a root element corresponding to the operation's input message
root = bds.createChildElement('NDFDgen')

# Create a map from the message part name to the value to use for that
# part.
request_values = { 'latitude' : lat
                 , 'longitude' : lon
                 , 'startTime' : today
                 , 'endTime' : later
                 , 'product' : ndfd.productType.time_series
                 , 'Unit' : ndfd.unitType.e
                 , 'weatherParameters' : weather_params }

# Get the WSDL message description, and for each part for which we
# have a value, add an element to the message document.
req_msg = spec.messageMap()['NDFDgenRequest']
for p in req_msg.part:
    fv = request_values.get(p.name)
    if fv is None:
        # Fatal error if a field required by the message is not available
        raise Exception('%s: %s has no value' % (p.name, p.typeReference.expandedName()))
    else:
        print('%s: %s' % (p.name, p.typeReference.expandedName()))
        # Make sure the value is of the expected type
        type_binding = p.typeReference.expandedName().typeBinding()
        if not isinstance(fv, type_binding):
            fv = type_binding.Factory(fv)
        fv.toDOM(bds, root, element_name=p.name)

# Finish the request message and get it as a DOM document.
dom = bds.finalize()

# We don't have a facility to add DOM values (as opposed to binding
# instances) to a soap binding instance, so just directly generate the
# message by wedging the request into a generic SOAP envelope body.
soap_message = '''<?xml version="1.0" encoding="ISO-8859-1"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
  <SOAP-ENV:Body>''' + dom.documentElement.toxml("utf-8") + '''</SOAP-ENV:Body>
</SOAP-ENV:Envelope>'''

#soap_message = open('NDFDgen.xml').read()
#soap_message = open('test.xml').read()
# Save the request message so it can be examined later
open('req.xml', 'w').write(soap_message)

# Pull the SOAPAction and endpoint out of the WSDL spec.  This is gross.
spec_ns = spec.namespaceContext().targetNamespace()
binding = spec_ns.createExpandedName('ndfdXMLBinding').binding()
operation = binding.operationMap()['NDFDgen']
soap_op = operation.wildcardElements()[0]
soap_action = soap_op.soapAction

service = spec_ns.createExpandedName('ndfdXML').service()
soap_addr = service.port[0].wildcardElements()[0]
endpoint = soap_addr.location

# Execute the request
uri = urllib_request.Request(endpoint,
                      soap_message,
                      { 'SOAPAction' : soap_action, 'Content-Type': 'text/xml' } )
rxml = urllib_request.urlopen(uri).read()
#rxml = open('rawresp.xml').read()

# Save the raw SOAP-wrapped response
open('rawresp.xml', 'w').write(rxml)

# The NDFD interface is "interesting" in that the response message for
# the SOAP interface is encoded as a text string, rather than being
# provided as XML directly.  The noise below extracts it.
rdom = domutils.StringToDOM(rxml)
resp = soapenv.CreateFromDOM(rdom)
v = resp.Body.wildcardElements()[0]
rxml = v.childNodes[0].childNodes[0].value
# Save the extracted response
open('resp.xml', 'w').write(rxml)
#rxml = open('resp.xml').read()

# Create the binding instance from the response.  If there's a
# problem, diagnose the issue, then try again with validation
# disabled.
r = None
try:
    r = DWML.CreateFromDocument(rxml)
except pyxb.UnrecognizedContentError as e:
    print('*** ERROR validating response:')
    print(e.details())
if r is None:
    pyxb.RequireValidWhenParsing(False)
    r = DWML.CreateFromDocument(rxml)

# Start spitting out the processed data.
product = r.head.product
print('%s %s' % (product.title, product.category))
source = r.head.source
print(", ".join(source.production_center.content()))
data = r.data[0]

for i in range(len(data.location)):
    loc = data.location[i]
    print('%s [%s %s]' % (loc.location_key, loc.point.latitude, loc.point.longitude))
    for p in data.parameters:
        if p.applicable_location != loc.location_key:
            continue
        mint = maxt = None
        for t in p.temperature:
            if 'maximum' == t.type:
                maxt = t
            elif 'minimum' == t.type:
                mint = t
            print('%s (%s): %s' % (t.name, t.units, " ".join([ str(_v.value()) for _v in t.value_ ])))
        time_layout = None
        for tl in data.time_layout:
            if tl.layout_key == mint.time_layout:
                time_layout = tl
                break
        for ti in range(len(time_layout.start_valid_time)):
            start = time_layout.start_valid_time[ti].value()
            end = time_layout.end_valid_time[ti]
            print('%s: min %s, max %s' % (time.strftime('%A, %B %d %Y', start.timetuple()),
                                          mint.value_[ti].value(), maxt.value_[ti].value()))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.fac
import sys

import os.path
xsd='''
<xsd:schema xmlns:xsd="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified" attributeFormDefault="unqualified">
  <!-- Essentially: (a*)* -->
  <xsd:element name="a" type="xsd:string"/>
  <xsd:complexType name="parametersType">
    <xsd:sequence minOccurs="0" maxOccurs="unbounded">
      <xsd:choice>
        <xsd:element ref="a" minOccurs="0" maxOccurs="unbounded"/>
      </xsd:choice>
    </xsd:sequence>
  </xsd:complexType>
  <xsd:element name="parameters" type="parametersType"/>
</xsd:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0172 (unittest.TestCase):
    def testBase (self):
        instance = parametersType()
        cfg = instance._automatonConfiguration()
        self.assertEqual(1, cfg.nondeterminismCount())
        # Step once
        a_ed = parametersType._UseForTag('a')
        cfg.step('a', a_ed)
        self.assertEqual(1, cfg.nondeterminismCount())
        # There are two ways to re-enter a: loop within a, or exit the
        # choice and re-enter.  Same destination, same element
        # declaration, two update instruction sets.
        cfg.step('a', a_ed)
        self.assertEqual(2, cfg.nondeterminismCount())
        # But there's a problem here, which is now trac/173
        cfg.step('a', a_ed)
        self.assertEqual(4, cfg.nondeterminismCount())
        self.assertTrue(4 < cfg.PermittedNondeterminism)
        cfg.PermittedNondeterminism = 4
        with self.assertRaises(pyxb.ContentNondeterminismExceededError) as cm:
            cfg.step('a', a_ed)

if __name__ == '__main__':
    unittest.main()
